const request = require('supertest');
const app = require('./app.js');


afterAll(() => {
    app.closeFirebase();
});

describe('POST /templates', () => {

    test('Calling with a valid templates succeeds', async () => {
        const response = await
            request(app).post('/templates').send({
                "Doge": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "South",
                                "x": 0,
                                "y": 20
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "North",
                                "x": 0,
                                "y": 50
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }
                }

            });
        var body = response.body;
        expect(response.body).toEqual({status: "OK"})
        expect(response.statusCode).toBe(200);
    });
    test('Calling without a text block gives a 400 error code', async () => {
        const response = await
            request(app).post('/templates').send({
                "Doge": {}

            });
        var body = response.body;
        expect(response.body.status).toEqual("ERROR")
        expect(response.statusCode).toBe(400);
    });
    test('Calling with invalid properties in a text block gives a 400 error code', async () => {

        //No fillColor
        let response = await
            request(app).post('/templates').send({
                "Doge": {
                    "text": {
                        "bottom": {
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "South",
                                "x": 0,
                                "y": 20
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "North",
                                "x": 0,
                                "y": 50
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }


                }
            });
        expect(response.body.status).toEqual("ERROR")
        expect(response.statusCode).toBe(400);

        //No font
        response = await
            request(app).post('/templates').send({
                "Doge": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "location": {
                                "gravity": "South",
                                "x": 0,
                                "y": 20
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "location": {
                                "gravity": "North",
                                "x": 0,
                                "y": 50
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }


                }
            });
        expect(response.body.status).toEqual("ERROR")
        expect(response.statusCode).toBe(400);

        //No location
        response = await
            request(app).post('/templates').send({
                "Doge": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }


                }
            });
        expect(response.body.status).toEqual("ERROR")
        expect(response.statusCode).toBe(400);

        //No size
        response = await
            request(app).post('/templates').send({
                "Doge": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "South",
                                "x": 0,
                                "y": 20
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "North",
                                "x": 0,
                                "y": 50
                            },
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }


                }
            });
        expect(response.body.status).toEqual("ERROR")
        expect(response.statusCode).toBe(400);

        //No strokeWidth
        response = await
            request(app).post('/templates').send({
                "Doge": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "South",
                                "x": 0,
                                "y": 20
                            },
                            "size": 54,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "North",
                                "x": 0,
                                "y": 50
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }


                }
            });
        expect(response.body.status).toEqual("ERROR");
        expect(response.statusCode).toBe(400);

        //No strokeColor
        response = await request(app).post('/templates').send({
            "Doge": {
                "text": {
                    "bottom": {
                        "fillColor": "#FFF",
                        "font": "Impact.ttf",
                        "location": {
                            "gravity": "South",
                            "x": 0,
                            "y": 20
                        },
                        "size": 54,
                        "strokeWidth": 1.5
                    },
                    "top": {
                        "fillColor": "#FFF",
                        "font": "Impact.ttf",
                        "location": {
                            "gravity": "North",
                            "x": 0,
                            "y": 50
                        },
                        "size": 54,
                        "strokeWidth": 1.5,
                        "strokeColor": "#000"
                    }
                }


            }
        });
        expect(response.body.status).toEqual("ERROR");
        expect(response.statusCode).toBe(400);


        //No location.x
        response = await
            request(app).post('/templates').send({
                "Doge": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "South",
                                "y": 20
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "North",
                                "y": 50
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }


                }
            });
        expect(response.body.status).toEqual("ERROR");
        expect(response.statusCode).toBe(400);

        //No location.y
        response = await
            request(app).post('/templates').send({
                "Doge": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "South",
                                "x": 0
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "North",
                                "x": 0
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }


                }
            });
        expect(response.body.status).toEqual("ERROR")
        expect(response.statusCode).toBe(400);

        //No location.gravity
        response = await
            request(app).post('/templates').send({
                "Doge": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "x": 0,
                                "y": 20
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "x": 0,
                                "y": 50
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }


                }
            });
        expect(response.body.status).toEqual("ERROR")
        expect(response.statusCode).toBe(400);
    });
});

describe("GET /templates", () => {
    test("Lists all templates (also tests POST)", async () => {
        let response = await
            request(app).post('/templates').send({
                "Doge": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "South",
                                "x": 0,
                                "y": 20
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "North",
                                "x": 0,
                                "y": 50
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }
                }

            });
        expect(response.body).toEqual({status: "OK"})
        expect(response.statusCode).toBe(200);

        response = await request(app).get("/templates");
        expect(response.statusCode).toBe(200);
    });
    test("/templates/:name returns a single templates well-formed (also tests POST)", async () => {
            let response = await
                request(app).post('/templates').send({
                    "Doge": {
                        "text": {
                            "bottom": {
                                "fillColor": "#FFF",
                                "font": "Impact.ttf",
                                "location": {
                                    "gravity": "South",
                                    "x": 0,
                                    "y": 20
                                },
                                "size": 54,
                                "strokeWidth": 1.5,
                                "strokeColor": "#000"
                            },
                            "top": {
                                "fillColor": "#FFF",
                                "font": "Impact.ttf",
                                "location": {
                                    "gravity": "North",
                                    "x": 0,
                                    "y": 50
                                },
                                "size": 54,
                                "strokeWidth": 1.5,
                                "strokeColor": "#000"
                            }
                        }
                    }

                });
            expect(response.body).toEqual({status: "OK"})
            expect(response.statusCode).toBe(200);

            response = await request(app).get("/templates/Doge");
            expect(response.statusCode).toBe(200);
            if(response.body && response.body.Doge)
                delete response.body.Doge.count;
            expect(response.body).toEqual({
                "Doge": {
                    "text": {
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {"gravity": "North", "x": 0, "y": 50},
                            "size": 54,
                            "strokeColor": "#000",
                            "strokeWidth": 1.5
                        },
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {"gravity": "South", "x": 0, "y": 20},
                            "size": 54,
                            "strokeColor": "#000",
                            "strokeWidth": 1.5
                        }
                    }
                }
            });
        }
    )

})
describe('PUT /templates', () => {
    test("Uploads a file to the server", async () => {

        //first, make sure that a Doge exists
        let response = await
            request(app).post('/templates').send({
                "Doge3": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "South",
                                "x": 0,
                                "y": 20
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "North",
                                "x": 0,
                                "y": 50
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }
                }

            });
        expect(response.body).toEqual({status: "OK"})
        expect(response.statusCode).toBe(200);

        //Try to call the PUT
        response = await request(app).put("/templates/Doge3").attach("file", "resources/doge.jpg");
        expect(response.statusCode).toBe(200);
    });
    test("Updates the templates's graphics field (also tests GET /templates/:name)", async () => {
        let response = await
            request(app).post('/templates').send({
                "Doge1": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "South",
                                "x": 0,
                                "y": 20
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "North",
                                "x": 0,
                                "y": 50
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }
                }

            });
        expect(response.body).toEqual({status: "OK"})
        expect(response.statusCode).toBe(200);

        //Try to call the PUT
        response = await request(app).put("/templates/Doge1").attach("file", "resources/doge.jpg");
        expect(response.body).toEqual({status: "OK"})
        expect(response.statusCode).toBe(200);

        //Look to see if the templates's graphics is now correct
        response = await request(app).get("/templates/Doge1");
        expect(response.statusCode).toBe(200);
        if(response.body && response.body.Doge1)
            delete response.body.Doge1.count;

        expect(response.body).toEqual({
            "Doge1": {
                "graphic": "dbce44312027ca7f16631937e62ff5e9",
                "text": {
                    "bottom": {
                        "fillColor": "#FFF",
                        "font": "Impact.ttf",
                        "location": {
                            "gravity": "South",
                            "x": 0,
                            "y": 20
                        },
                        "size": 54,
                        "strokeWidth": 1.5,
                        "strokeColor": "#000"
                    },
                    "top": {
                        "fillColor": "#FFF",
                        "font": "Impact.ttf",
                        "location": {
                            "gravity": "North",
                            "x": 0,
                            "y": 50
                        },
                        "size": 54,
                        "strokeWidth": 1.5,
                        "strokeColor": "#000"
                    }
                }
            }

        });

    });
    test("Returns 400 error when templates doesn't exist", async () => {
        let response = await request(app).put("/templates/non-existant-templates" + Math.random() + "-" + Date.now()).attach("file", "resources/doge.jpg");
        expect(response.statusCode).toBe(400);
    });
    test("Returns 400 error when no file is uploaded", async () => {
        let response = await
            request(app).post('/templates').send({
                "Doge2": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "South",
                                "x": 0,
                                "y": 20
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "North",
                                "x": 0,
                                "y": 50
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }
                }

            });
        expect(response.body).toEqual({status: "OK"})
        expect(response.statusCode).toBe(200);
        response = await request(app).put("/templates/doge2");
        expect(response.statusCode).toBe(400);
    });
});

describe("DELETE and integration", () => {
    test("POST DELETE GET results in no new templates added", async () => {

        let response = await
            request(app).post('/templates').send({
                "Doge": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "South",
                                "x": 0,
                                "y": 20
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "North",
                                "x": 0,
                                "y": 50
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }
                }

            });
        expect(response.body).toEqual({status: "OK"})
        expect(response.statusCode).toBe(200);

        response = await request(app).delete("/templates/Doge");
        expect(response.body).toEqual({status: "OK"})
        expect(response.statusCode).toBe(200);

        response = await request(app).get("/templates/Doge");
        expect(response.body).toEqual({});
        expect(response.statusCode).toBe(200);

    });

    test("DELETE an invalid meme returns error 400", async () => {
        let response = await request(app).delete("/templates/NonExistanttemplates" + Date.now());
        expect(response.body.status).toEqual("ERROR")
        expect(response.statusCode).toBe(400);
    });
});

