const request = require('supertest');
const app = require('./app.js');


afterAll(() => {
    app.closeFirebase();
});

describe("Generates memes", () => {
    test("Using a valid template succeeds", async () => {

        let response = await
            request(app).post('/templates').send({
                "Doge9": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "South",
                                "x": 0,
                                "y": 20
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "North",
                                "x": 0,
                                "y": 50
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }
                }

            });
        var body = response.body;
        expect(response.body).toEqual({status: "OK"})
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(200);
        response = await request(app).put("/templates/Doge9").attach("file", "resources/doge.jpg")
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(200);

        //Now that we are sure the template exists, try to make the meme
        response = await request(app).post("/memes").send({
            memeTemplate: "Doge9",
            text: {
                top: "Doge cries",
                bottom: "When tests fail"
            }
        });
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(200);
    });
    test("Request with an invalid template returns 400", async () => {

        let response = await request(app).post("/memes").send({
            memeTemplate: "nonExistantTemplate" + Date.now(),
            text: {
                step1: "Make a new NPM\nProject",
                step2: "Generate lots\nof memes",
                step3: "Forget to actually\ndo the\nassignment",
                step4: "Forget to actually\ndo the\nassignment"
            }
        });

        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(400);
    });
    test("Request with missing text returns 400", async () => {
        let response = await request(app).post("/memes").send({
            memeTemplate: "Gru"
        });
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(400);
    });
    test("Using a template that doesn't have a graphic returns 400", async () => {
        let response = await
            request(app).post('/templates').send({
                "Doge": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "South",
                                "x": 0,
                                "y": 20
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "North",
                                "x": 0,
                                "y": 50
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }
                }

            });
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(200);
        var body = response.body;
        expect(response.body).toEqual({status: "OK"})

        response = await request(app).post("/memes").send({
            memeTemplate: "Doge",
            text: {
                top: "Doge",
                bottom: "Such meme",
            }
        });
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(400);
    });
    test("Generating a meme increments the count for that template", async () => {
        let response = await
            request(app).post('/templates').send({
                "Doge00": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "South",
                                "x": 0,
                                "y": 20
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "North",
                                "x": 0,
                                "y": 50
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }
                }

            });
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(200);
        var body = response.body;
        expect(response.body).toEqual({status: "OK"})
        response = await request(app).put("/templates/Doge00").attach("file", "resources/doge.jpg")
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(200);

        //Now that we are sure the template exists, try to make the meme
        response = await request(app).post("/memes").send({
            memeTemplate: "Doge00",
            text: {
                top: "Doge cries",
                bottom: "When tests fail"
            }
        });
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(200);

        response = await request(app).post("/memes").send({
            memeTemplate: "Doge00",
            text: {
                top: "Doge cries",
                bottom: "When tests are flaky"
            }
        });
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(200);

        response = await request(app).get("/templates/Doge00");
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(200);
        expect(response.body.Doge00.count).toBe(2);
    });
    test("Failed meme generation does NOT increment the counter for that template", async () => {
        let response = await
            request(app).post('/templates').send({
                "Doge01": {
                    "text": {
                        "bottom": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "South",
                                "x": 0,
                                "y": 20
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        },
                        "top": {
                            "fillColor": "#FFF",
                            "font": "Impact.ttf",
                            "location": {
                                "gravity": "North",
                                "x": 0,
                                "y": 50
                            },
                            "size": 54,
                            "strokeWidth": 1.5,
                            "strokeColor": "#000"
                        }
                    }
                }

            });
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(200);
        var body = response.body;
        expect(response.body).toEqual({status: "OK"})
        response = await request(app).put("/templates/Doge01").attach("file", "resources/doge.jpg")
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(200);

        response = await request(app).post("/memes").send({
            memeTemplate: "Doge01",
            text: {
                top: "Doge cries",
                bottom: "When tests fail"
            }
        });
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(200);
        response = await request(app).post("/memes").send({
            memeTemplate: "Doge01"
        });
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(400);

        response = await request(app).get("/templates/Doge01");
        if(response.statusCode == 500)
        {
            console.log("Unexpected error on server:");
            console.log(response.body);
        }
        expect(response.statusCode).toBe(200);
        expect(response.body.Doge01.count).toBe(1);
    });

})
